Crate piece_tree

Crate piece_tree 

Macros

entity_impl
Macro which provides the common implementation of a 32-bit entity reference.
smallvec
Creates a [SmallVec] containing the arguments.

Structs

BufferRef
Buffers
ChunkIter
A lazy, non-allocating slice view over a byte range of the tree
HistoryEntry
LinesIter
MemoryUsage
Node
NodeRef
OriginalBuffer
Piece
PieceTree
PieceTreeIter
Pieces
ReverseTreeWalker
SliceChars
Non-allocating char iterator over a byte-bounded window of the tree.
TreeSlice
A zero-copy borrowed view over a byte range of the tree. All offsets are relative to the slice start.
TreeWalker

Enums

Color
Edit

Constants

CHECKPOINT_INTERVAL
MAX_PIECE_SIZE
MOD_BUFFER
NIL

Functions

assert_coordinates
assert_invariants
assert_no_mergeable_neighbors
assert_piece_metadata
assert_state
count_chars_and_newlines
count_chars_and_newlines_with_offsets_and_checkpoints